SystemOrganization addCategory: #'Limbo-Tests'!

TestCase subclass: #LimboDup2Test
	instanceVariableNames: 'pipe2 pipe1'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Limbo-Tests'

	classInstanceVariableNames:''!

!LimboDup2Test methodsFor: 'running' stamp: 'MML 11/23/2012 11:14'!
setUp
	pipe1 := LimboPipe new.
	pipe2 := LimboPipe new! !

!LimboDup2Test methodsFor: 'running' stamp: 'MML 11/23/2012 11:15'!
tearDown
	pipe1
		closeWriteEnd;
		closeReadEnd.
	pipe2
		closeWriteEnd;
		closeReadEnd! !

!LimboDup2Test methodsFor: 'tests' stamp: 'MML 11/23/2012 13:33'!
testDup2	
	| limbo string read |
	self
		shouldnt: [ limbo := LimboDup2 redirect: pipe2 writeFileDescriptor to: pipe1 writeFileDescriptor ]
		raise: Error.
	self assert: limbo exitStatus isZero.
	
	self
		shouldnt: [ limbo := LimboDup2 redirect: pipe2 readFileDescriptor to: pipe1 readFileDescriptor ]
		raise: Error.
	self assert: limbo exitStatus isZero.
	
	string := 'from 1 to 2'.
	pipe1 write: string.
	read := pipe2 read: string size.
	self assert: read equals: string.
	"cannot read multiple times. otherwise, read will block until there is more data in the buffer"
	pipe1 write: string.
	read := pipe1 read: string size.
	self assert: read equals: string.
	
	string := 'from 2 to 1'.
	pipe2 write: string.
	read := pipe2 read: string size.
	self assert: read equals: string.
	pipe1 write: string.
	read := pipe1 read: string size.
	self assert: read equals: string! !

TestCase subclass: #LimboFStatTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Limbo-Tests'

	classInstanceVariableNames:''!

!LimboFStatTest methodsFor: 'tests' stamp: 'MML 11/23/2012 10:52'!
testFStat
	| statStruct pipe string fstat |
	pipe := LimboPipe new.
	string := 'this is a test string'.
	pipe write: string.
	pipe closeWriteEnd.
	self
		shouldnt: [ fstat := LimboFStat stat: pipe readFileDescriptor ]
		raise: Error.
	pipe closeReadEnd.
	
	self assert: fstat statStruct st_size equals: string size + 1. "0 terminated"
	fstat free! !

TestCase subclass: #LimboForkTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Limbo-Tests'

	classInstanceVariableNames:''!

!LimboForkTest methodsFor: 'helpers' stamp: 'MML 11/23/2012 15:02'!
getPids
	| limbo vmPid stream lines |
	limbo := Limbo runSh: 'ps -Ao "pid, command" | grep -i cog'.
	self assert: limbo succeeded.
	
	vmPid := Integer 
		readFrom: (limbo outString readStream upTo: Character space)
		base: 10.
		
	limbo := Limbo runSh: 'ps -Ao "ppid, pid, command" | grep "^\s*', vmPid asString, '"'.
	self assert: limbo succeeded.
	
	lines := limbo outString findTokens: Character lf.
	lines := lines reject: [ :string | string includesSubString: 'grep' ].
	self assert: lines size equals: 1.
	
	stream := lines first readStream.
	stream upTo: Character space.
	^ Integer
		readFrom: (stream upTo: Character space)
		base: 10! !

!LimboForkTest methodsFor: 'tests' stamp: 'MML 11/22/2012 10:45'!
testFork
	| limbo pids |
	self
		shouldnt: [ limbo := LimboFork new ]
		raise: Error.
		
	pids := self getPids.! !

TestCase subclass: #LimboKillTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Limbo-Tests'

	classInstanceVariableNames:''!

!LimboKillTest methodsFor: 'helpers' stamp: 'MML 11/23/2012 15:01'!
checkIfKilled: aProcessId
	| limbo vmPid lines stream |
	limbo := Limbo runSh: 'ps -Ao "pid, command" | grep -i cog'.
	self assert: limbo succeeded.
	lines := limbo outString findTokens: Character lf.
	lines := lines reject: [ :string | string includesSubString: 'grep' ].
	self assert: lines size equals: 1! !

!LimboKillTest methodsFor: 'helpers' stamp: 'MML 11/23/2012 15:02'!
getChildProcessId
	| limbo vmPid lines stream |
	limbo := Limbo runSh: 'ps -Ao "pid, command" | grep -i cog'.
	self assert: limbo succeeded.
	
	vmPid := Integer 
		readFrom: (limbo outString readStream upTo: Character space)
		base: 10.
		
	limbo := Limbo runSh: 'ps -Ao "ppid, pid, command" | grep "^\s*', vmPid asString, '"'.
	self assert: limbo succeeded.
	
	lines := limbo outString findTokens: Character lf.
	lines := lines reject: [ :string | (string includesSubString: 'grep') ].
	self assert: lines size equals: 1.
	stream := lines first readStream.
	stream upTo: Character space.
	^ Integer
		readFrom: (stream upTo: Character space)
		base: 10! !

!LimboKillTest methodsFor: 'tests' stamp: 'MML 11/23/2012 14:56'!
testKillAndWait
	| fork kill childProcessId wait |
	fork := LimboFork new.
	childProcessId := self getChildProcessId.
	self
		shouldnt: [ kill := LimboKill kill: childProcessId ]
		raise: Error.
	self assert: kill exitStatus isZero.
	wait := LimboWait waitFor: childProcessId.
	self assert: wait exitStatus isZero.
	self checkIfKilled: childProcessId! !

TestCase subclass: #LimboPipeTest
	instanceVariableNames: 'pipe'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Limbo-Tests'

	classInstanceVariableNames:''!

!LimboPipeTest methodsFor: 'running' stamp: 'MML 11/22/2012 17:44'!
setUp
	pipe := LimboPipe new! !

!LimboPipeTest methodsFor: 'running' stamp: 'MML 11/22/2012 17:44'!
tearDown
	pipe
		closeReadEnd;
		closeWriteEnd! !

!LimboPipeTest methodsFor: 'tests' stamp: 'MML 11/21/2012 15:02'!
testPipe
	| pipe |
	self 
		shouldnt: [ pipe := LimboPipe new ]
		raise: Error.	
	self assert: pipe exitStatus isZero.	
	self assert: (pipe isKindOf: LimboPipe).
	self assert: pipe readFDPointer isNil.
	self assert: pipe writeFDPointer isNil.
	self assert: (pipe readFileDescriptor isKindOf: SmallInteger).
	self assert: (pipe writeFileDescriptor isKindOf: SmallInteger).! !

!LimboPipeTest methodsFor: 'tests' stamp: 'MML 11/22/2012 10:02'!
testPipeClose
	| pipe |
	pipe := LimboPipe new.
	
	self
		shouldnt: [ pipe closeReadEnd ] 
		raise: Error.
	self assert: pipe exitStatus isZero.
	
	self
		shouldnt: [ pipe closeWriteEnd ]
		raise: Error.
	self assert: pipe exitStatus isZero! !

!LimboPipeTest methodsFor: 'tests' stamp: 'MML 11/22/2012 17:52'!
testPipeRead
	| string out |
	string := 'this is a test string'.
	pipe write: string.
	
	self
		shouldnt: [ out := pipe read: string size ]
		raise: Error.
	self assert: pipe exitStatus isZero.
	self assert: out equals: string! !

!LimboPipeTest methodsFor: 'tests' stamp: 'MML 11/22/2012 17:43'!
testPipeWrite
	| string |
	string := 'this is a test string'.
	self
		shouldnt: [ pipe write: string ] 
		raise: Error.
	self assert: pipe exitStatus isZero.! !

TestCase subclass: #LimboTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Limbo-Tests'

	classInstanceVariableNames:''!

!LimboTest methodsFor: 'tests' stamp: 'MML 11/15/2012 16:07'!
testBasicCommand
	| arg limbo |
	arg := 'this is a test string', String crlf, 'with new lines'.
	self
		shouldnt: [ limbo := Limbo run: '/bin/echo' arguments: { arg } ]
		raise: LimboError.
		
	self assert: limbo notNil.
	self assert: limbo errorString isEmpty.
	self assert: limbo outString equals: arg, String lf! !

!LimboTest methodsFor: 'tests' stamp: 'MML 11/15/2012 15:22'!
testBasicShellCommand
	| arg limbo |
	arg := 'this is a test string'.
	self
		shouldnt: [ limbo := Limbo runSh: '/bin/echo ', arg  ]
		raise: LimboError.
		
	self assert: limbo errorString isEmpty.
	self assert: limbo outString equals: arg, String lf! !

!LimboTest methodsFor: 'tests' stamp: 'MML 11/19/2012 18:29'!
testCommandWithErrors
	| arg limbo |
	arg := 'foo'.
	self
		shouldnt: [ limbo := Limbo run: '/bin/cat' arguments: { arg } ]
		raise: LimboError.
		
	self assert: limbo notNil.
	self assert: limbo outString isEmpty.
	self deny: limbo errorString isEmpty.
	self assert: limbo errorString equals: 'cat: foo: No such file or directory', String lf! !

!LimboTest methodsFor: 'tests' stamp: 'MML 11/19/2012 18:54'!
testComplicatedCommand
	| limbo echo sed |
	limbo := Limbo run: '/usr/bin/which' arguments: { 'echo' }.
	self assert: limbo succeeded.
	echo := limbo outString allButLast.
	
	limbo := Limbo run: '/usr/bin/which' arguments: { 'sed' }.
	self assert: limbo succeeded.
	sed := limbo outString allButLast.
	
	limbo := Limbo runSh: echo, ' day | ', sed, ' s/day/night/'.
	self assert: limbo succeeded.
	self assert: limbo errorString isEmpty.
	self deny: limbo outString isEmpty.
	self assert: limbo outString equals: 'night', String lf! !

!LimboTest methodsFor: 'tests' stamp: 'MML 11/19/2012 18:28'!
testShellCommandWithErrors
	"passing a new line to /bin/sh will cause it to think that the arguments list is finished"
	| arg limbo |
	arg := 'this is a test string', String crlf, 'with new lines'.
	self
		shouldnt: [ limbo := Limbo runSh: '/bin/echo ' , arg  ]
		raise: LimboError.
		
	self deny: limbo errorString isEmpty.
	self assert: limbo errorString equals: '/bin/sh: line 1: with: command not found', String lf! !

!LimboTest methodsFor: 'tests' stamp: 'MML 11/19/2012 19:00'!
testStringArgument
	self
		should: [ Limbo run: 'foo' arguments: 'bar' ]
		raise: LimboError
		whoseDescriptionIncludes: 'expected collection of strings'
		description: 'expected an error because of the arguments being a string instead of a collection of strings'! !

!LimboTest methodsFor: 'tests' stamp: 'MML 11/19/2012 18:56'!
testSuccess
	| limbo |
	limbo := Limbo run: '/bin/cat' arguments: { 'foo' }.
	self deny: limbo succeeded.
	self deny: limbo command exitStatus isZero.
	self assert: limbo outString isEmpty.
	self deny: limbo errorString isEmpty! !
